@@ -4,8 +4,8 @@ import random
 import secrets
 import threading
 import time
-from urllib.parse import urlparse
 
+from urllib.parse import urlparse
 
 from ..helpers import deep_update
 from ..persistence import read_json_file, write_json_file
@@ -19,10 +19,9 @@ class ConfigManager(GenericDataManager):
     """
     The ConfigManager manages the configuration persisted in config.json
     It's not suppose to have any side-effects. Setting and getting only
-    with a lot of validation and computing while setting/getting
+    with a lot of validation and computing while setting/getting.
     """
 
-    initial_data = {}
     name_of_json_file = "config.json"
     lock = threading.Lock()
 
@@ -43,6 +42,10 @@ class ConfigManager(GenericDataManager):
                 "regtest": "CUSTOM",
                 "signet": "CUSTOM",
             },
+            # user-defined asset labels for liquid
+            "asset_labels": {
+                "liquidv1": {},
+            },
             "active_node_alias": "default",
             "proxy_url": "socks5h://localhost:9050",  # Tor proxy URL
             "only_tor": False,
@@ -59,12 +62,16 @@ class ConfigManager(GenericDataManager):
             "price_provider": "",
             "weight_unit": "oz",
             "validate_merkle_proofs": False,
-            "fee_estimator": "mempool",
+            "fee_estimator": "bitcoin_core",
             "fee_estimator_custom_url": "",
             "hide_sensitive_info": False,
+            "autohide_sensitive_info_timeout_minutes": 20,
+            "autologout_timeout_hours": 4,
             # TODO: remove
             "bitcoind": False,
         }
+
+        # TODO: change this to just override load()
         self.check_config()
 
     def check_config(self):
@@ -87,7 +94,7 @@ class ConfigManager(GenericDataManager):
             # unique id of specter
             if self.data["uid"] == "":
                 self.config["uid"] = (
-                    random.randint(0, 256 ** 8).to_bytes(8, "big").hex()
+                    random.randint(0, 256**8).to_bytes(8, "big").hex()
                 )
             self._save()
 
@@ -110,6 +117,19 @@ class ConfigManager(GenericDataManager):
             auth["registration_link_timeout"] = registration_link_timeout
         self._save()
 
+    def update_asset_label(self, asset, label, chain, user):
+        if user.is_admin:
+            if "asset_labels" not in self.data:
+                self.data["asset_labels"] = {}
+            if not label:
+                if self.data["asset_labels"].get(chain, {}).get(asset):
+                    del self.data["asset_labels"][chain][asset]
+            else:
+                deep_update(self.data["asset_labels"], {chain: {asset: label}})
+            self._save()
+        else:
+            user.update_asset_label(asset, label, chain)
+
     def update_explorer(self, explorer_id, explorer_data, user, chain):
         """update the block explorers urls"""
         if isinstance(user, str):
@@ -160,6 +180,12 @@ class ConfigManager(GenericDataManager):
             self.data["proxy_url"] = proxy_url
             self._save()
 
+    def update_tor_type(self, tor_type, user):
+        """update the Tor type to use"""
+        if self.data.get("tor_type", "builtin") != tor_type:
+            self.data["tor_type"] = tor_type
+            self._save()
+
     def toggle_tor_status(self):
         """toggle the Tor status"""
         self.data["tor_status"] = not self.data["tor_status"]
@@ -245,6 +271,24 @@ class ConfigManager(GenericDataManager):
         else:
             user.set_hide_sensitive_info(hide_sensitive_info_bool)
 
+    def update_autohide_sensitive_info_timeout(self, timeout_minutes, user):
+        if isinstance(user, str):
+            raise Exception("Please pass a real user, not a string-user")
+        if user.is_admin:
+            self.data["autohide_sensitive_info_timeout_minutes"] = timeout_minutes
+            self._save()
+        else:
+            user.set_autohide_sensitive_info_timeout(timeout_minutes)
+
+    def update_autologout_timeout(self, timeout_hours, user):
+        if isinstance(user, str):
+            raise Exception("Please pass a real user, not a string-user")
+        if user.is_admin:
+            self.data["autologout_timeout_hours"] = timeout_hours
+            self._save()
+        else:
+            user.set_autologout_timeout(timeout_hours)
+
     def update_price_provider(self, price_provider, user):
         if isinstance(user, str):
             raise Exception("Please pass a real user, not a string-user")
