@@ -1,9 +1,10 @@
 import random, traceback
+from time import time
 from binascii import unhexlify
 from flask import make_response
 from flask_wtf.csrf import CSRFError
-from werkzeug.exceptions import MethodNotAllowed
-from flask import render_template, request, redirect, url_for, flash
+from werkzeug.exceptions import MethodNotAllowed, NotFound
+from flask import render_template, request, redirect, url_for, flash, g
 from flask_babel import lazy_gettext as _
 from flask_login import login_required, current_user
 from ..helpers import (
@@ -31,15 +32,21 @@ from .price import price_endpoint
 from .settings import settings_endpoint
 from .setup import setup_endpoint
 from .wallets import wallets_endpoint
+from .wallets_api import wallets_endpoint_api
 from ..rpc import RpcError
 
+# Services live in their own separate path
+from cryptoadvance.specter.services.controller import services_endpoint
+
 app.register_blueprint(auth_endpoint, url_prefix="/auth")
 app.register_blueprint(devices_endpoint, url_prefix="/devices")
 app.register_blueprint(nodes_endpoint, url_prefix="/nodes")
 app.register_blueprint(price_endpoint, url_prefix="/price")
+app.register_blueprint(services_endpoint, url_prefix="/services")
 app.register_blueprint(settings_endpoint, url_prefix="/settings")
 app.register_blueprint(setup_endpoint, url_prefix="/setup")
 app.register_blueprint(wallets_endpoint, url_prefix="/wallets")
+app.register_blueprint(wallets_endpoint_api, url_prefix="/wallets")
 
 rand = random.randint(0, 1e32)  # to force style refresh
 
@@ -76,6 +83,14 @@ def server_specter_error(se):
         return redirect(url_for("about"))
 
 
+@app.errorhandler(NotFound)
+def server_notFound_error(e):
+    """Unspecific Exceptions get a 404 Error-Page"""
+    # if rpc is not available
+    app.logger.error("Could not find Resource (404): %s" % request.url)
+    return render_template("500.jinja", error=e), 500
+
+
 @app.errorhandler(Exception)
 def server_error(e):
     """Unspecific Exceptions get a 500 Error-Page"""
@@ -145,6 +160,33 @@ def selfcheck():
         app.login("admin")
 
 
+@app.before_request
+def slow_request_detection():
+    """ """
+    g.start = time()
+
+
+@app.after_request
+def after_request(response):
+    diff = time() - g.start
+    if (
+        (response.response)
+        and (200 <= response.status_code < 300)
+        and (response.content_type.startswith("text/html"))
+    ):
+        threshold = app.config["REQUEST_TIME_WARNING_THRESHOLD"]
+        if diff > threshold:
+            flash(
+                _(
+                    "The request before this one took {} seconds which is longer than the threshold ({}). Checkout the perfomance-improvement-hints in the documentation".format(
+                        int(diff), threshold
+                    )
+                ),
+                "warning",
+            )
+    return response
+
+
 ########## template injections #############
 @app.context_processor
 def inject_debug():
