@@ -15,6 +15,12 @@ def ascii20(context, name):
     return to_ascii20(name)
 
 
+@contextfilter
+@filters_bp.app_template_filter("unique_len")
+def unique_len(context, arr):
+    return len(set(arr))
+
+
 @contextfilter
 @filters_bp.app_template_filter("datetime")
 def timedatetime(context, s):
